# General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
# base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
# The model can simulate OLEDs, Perovskite cells, and OFETs.
# 
# Copyright (C) 2008-2020 Roderick C. I. MacKenzie
# All rights reserved.
# 
# https://www.gpvdm.com
# r.c.i.mackenzie at googlemail.com
# 
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the GPVDM nor the
#       names of its contributors may be used to endorse or promote products
#       derived from this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


import os

from scan_select import select_param
from token_lib import tokens
from scan_item import scan_items_get_list

from plot_io import plot_save_oplot_file
from scan_io import scan_push_to_hpc
from scan_io import scan_import_from_hpc
from cal_path import get_exe_command
from icon_lib import QIcon_load
from scan_item import scan_items_get_file
from scan_item import scan_items_get_token

from gpvdm_select import gpvdm_select


from util import str2bool

import i18n
_ = i18n.language.gettext

#qt
from PyQt5.QtCore import QSize, Qt
from PyQt5.QtWidgets import QWidget,QVBoxLayout,QToolBar,QSizePolicy,QAction,QTabWidget,QTableWidget,QAbstractItemView, QMenuBar, QTableWidgetItem
from PyQt5.QtGui import QPainter,QIcon

from gui_util import tab_add
from gui_util import tab_remove
from gui_util import tab_get_value

from inp import inp_save_lines_to_file
from inp import inp_load_file

from scan_item import scan_items_lookup_item
from cal_path import get_sim_path

from gui_util import tab_move_up
from gui_util import tab_move_down
from gui_util import tab_insert_row

class constraints(QWidget):

	def insert_row_mm(self,i,file_name,token,path,function,max_value,min_value,error):
		self.tab_mm.blockSignals(True)

		item = QTableWidgetItem(file_name)
		self.tab_mm.setItem(i,0,item)

		item = QTableWidgetItem(token)
		self.tab_mm.setItem(i,1,item)

		self.item = gpvdm_select()
		self.item.setText(path)
		self.item.button.clicked.connect(self.callback_show_list_src)
		self.tab_mm.setCellWidget(i,2,self.item)


		item = QTableWidgetItem(function)
		self.tab_mm.setItem(i,3,item)

		item = QTableWidgetItem(max_value)
		self.tab_mm.setItem(i,4,item)

		item = QTableWidgetItem(min_value)
		self.tab_mm.setItem(i,5,item)

		item = QTableWidgetItem(error)
		self.tab_mm.setItem(i,6,item)

		self.tab_mm.blockSignals(False)

	def insert_row_math(self,i,file_a,token_a,path_a,file_b,token_b,path_b,equation):
		self.tab_math.blockSignals(True)
		self.tab_math.insertRow(i)

		item = QTableWidgetItem(file_a)
		self.tab_math.setItem(i,0,item)

		item = QTableWidgetItem(token_a)
		self.tab_math.setItem(i,1,item)

		self.item = gpvdm_select()
		self.item.setText(path_a)
		self.item.button.clicked.connect(self.callback_show_list_equation_a)
		self.tab_math.setCellWidget(i,2,self.item)

		item = QTableWidgetItem(file_b)
		self.tab_math.setItem(i,3,item)

		item = QTableWidgetItem(token_b)
		self.tab_math.setItem(i,4,item)

		self.item = gpvdm_select()
		self.item.setText(path_b)
		self.item.button.clicked.connect(self.callback_show_list_equation_b)
		self.tab_math.setCellWidget(i,5,self.item)

		item = QTableWidgetItem(equation)
		self.tab_math.setItem(i,6,item)

		self.tab_math.blockSignals(False)

	def callback_show_list_equation_a(self):
		self.select_param_window_math_a.update()
		self.select_param_window_math_a.show()

	def callback_show_list_equation_b(self):
		self.select_param_window_math_b.update()
		self.select_param_window_math_b.show()

	def callback_show_list_src(self):
		self.select_param_window_mm.update()
		self.select_param_window_mm.show()

	#def callback_show_list_dest(self):
	#	self.select_param_window_dest.update()
	#	self.select_param_window_dest.show()

	def callback_add_item_mm(self):
		pos=tab_insert_row(self.tab_mm)
		self.insert_row_mm(pos,_("File"),_("Token"),_("Path"),_("Function"),_("Max"),_("Min"),_("Error"))
		self.save_combo()

	def callback_delete_item_mm(self):
		tab_remove(self.tab_mm)
		self.save_combo()

	def callback_add_item_math(self):
		self.insert_row_math(self.tab_math.rowCount(),_("File (a)"),_("Token (a)"),_("Path (a)"),_("File (b)"),_("Token (b)"),_("Path (b)"),_("Equation"))
		self.save_combo()

	def callback_delete_item_math(self):
		tab_remove(self.tab_math)
		self.save_combo()

	def save_combo(self):
		lines=[]
		for i in range(0,self.tab_mm.rowCount()):
			line="mm "+str(tab_get_value(self.tab_mm,i, 0))+" "+str(tab_get_value(self.tab_mm,i, 1))+" "+str(tab_get_value(self.tab_mm,i, 3))+" "+str(tab_get_value(self.tab_mm,i, 4))+" "+str(tab_get_value(self.tab_mm,i, 5))+" "+str(tab_get_value(self.tab_mm,i, 6))
			lines.append(line)

		for i in range(0,self.tab_math.rowCount()):
			line="math "+str(tab_get_value(self.tab_math,i, 0))+" "+str(tab_get_value(self.tab_math,i, 1))+" "+str(tab_get_value(self.tab_math,i, 3))+" "+str(tab_get_value(self.tab_math,i, 4))+" "+str(tab_get_value(self.tab_math,i, 6))
			lines.append(line)

		lines.append("#end")
		print("save as",self.file_name)
		inp_save_lines_to_file(self.file_name,lines)


	def tab_changed(self):
		print("here")
		self.save_combo()


	def create_model_mm(self):
		self.tab_mm.clear()
		self.tab_mm.setColumnCount(7)
		self.tab_mm.setSelectionBehavior(QAbstractItemView.SelectRows)
		self.tab_mm.setHorizontalHeaderLabels([ _("File"), _("Token"),_("Path"),_("Function"), _("Max"), _("Min"),_("Error")])
		self.tab_mm.setColumnWidth(2, 300)
		#self.tab_mm.setColumnWidth(5, 200)


		lines=[]
		pos=0
		lines=inp_load_file(self.file_name)
		if lines!=False:
			mylen=len(lines)
			while(1):

				if lines[pos]=="#end":
					break
				line=lines[pos].split()
				print(line)
				if line[0]=="mm":
					path=scan_items_lookup_item(line[1],line[2])
					self.tab_mm.insertRow(self.tab_mm.rowCount())
					self.insert_row_mm(self.tab_mm.rowCount()-1,line[1],line[2],path,line[3],line[4],line[5],line[6])

				pos=pos+1

	def create_model_math(self):
		self.tab_math.clear()
		self.tab_math.setColumnCount(7)
		self.tab_math.setSelectionBehavior(QAbstractItemView.SelectRows)
		#math fit_target2.inp #fxdomain_Ji ./sim/39.811Hz/sim_info.dat #fxdomain_Ji abs(20*(a-b))
		self.tab_math.setHorizontalHeaderLabels([ _("File (a)"), _("Token (a)"),_("Path (a)"),_("File (b)"),_("Token (b)"),_("Path (b)"), _("Equation")])

		lines=[]
		pos=0
		lines=inp_load_file(self.file_name)
		if lines!=False:
			mylen=len(lines)
			while(1):

				if lines[pos]=="#end":
					break
				line=lines[pos].split()
				if line[0]=="math":
					print(line)
					path_a=scan_items_lookup_item(line[1],line[2])
					path_b=scan_items_lookup_item(line[3],line[4])
					self.insert_row_math(self.tab_math.rowCount(),line[1],line[2],path_a,line[3],line[4],path_b,line[5])

				pos=pos+1

	def __init__(self):
		QWidget.__init__(self)
		self.file_name=os.path.join(get_sim_path(),"constraints.inp")
		self.setWindowTitle(_("Fit constraints window")+" - https://www.gpvdm.com")
		self.setWindowIcon(QIcon_load("constraints"))
		self.setFixedSize(900, 700)

		self.vbox=QVBoxLayout()

		########################mm##########################
		toolbar_mm=QToolBar()
		toolbar_mm.setIconSize(QSize(32, 32))

		self.tb_save = QAction(QIcon_load("list-add"), _("Add"), self)
		self.tb_save.triggered.connect(self.callback_add_item_mm)
		toolbar_mm.addAction(self.tb_save)

		self.tb_save = QAction(QIcon_load("list-remove"), _("Minus"), self)
		self.tb_save.triggered.connect(self.callback_delete_item_mm)
		toolbar_mm.addAction(self.tb_save)

		self.tb_down= QAction(QIcon_load("go-down"), _("Move down"), self)
		self.tb_down.triggered.connect(self.on_move_down)
		toolbar_mm.addAction(self.tb_down)

		self.tb_up= QAction(QIcon_load("go-up"), _("Move up"), self)
		self.tb_up.triggered.connect(self.on_move_up)
		toolbar_mm.addAction(self.tb_up)

		self.vbox.addWidget(toolbar_mm)

		self.tab_mm = QTableWidget()
		self.tab_mm.resizeColumnsToContents()

		self.tab_mm.verticalHeader().setVisible(False)
		self.create_model_mm()

		self.tab_mm.cellChanged.connect(self.tab_changed)

		self.select_param_window_mm=select_param()
		self.select_param_window_mm.init(self.tab_mm)
		self.select_param_window_mm.set_save_function(self.save_combo)
		self.select_param_window_mm.file_name_tab_pos=0
		self.select_param_window_mm.token_tab_pos=1
		self.select_param_window_mm.path_tab_pos=2


		self.vbox.addWidget(self.tab_mm)

		#####################math##########################
		toolbar_math=QToolBar()
		toolbar_math.setIconSize(QSize(32, 32))

		self.tb_save = QAction(QIcon_load("list-add"), _("Add"), self)
		self.tb_save.triggered.connect(self.callback_add_item_math)
		toolbar_math.addAction(self.tb_save)

		self.tb_save = QAction(QIcon_load("list-remove"), _("Minus"), self)
		self.tb_save.triggered.connect(self.callback_delete_item_math)
		toolbar_math.addAction(self.tb_save)

		self.vbox.addWidget(toolbar_math)
		self.tab_math = QTableWidget()
		self.tab_math.resizeColumnsToContents()

		self.tab_math.verticalHeader().setVisible(False)

		self.create_model_math()

		self.tab_math.cellChanged.connect(self.tab_changed)

		self.select_param_window_math_a=select_param()
		self.select_param_window_math_a.init(self.tab_math)
		self.select_param_window_math_a.set_save_function(self.save_combo)
		self.select_param_window_math_a.file_name_tab_pos=0
		self.select_param_window_math_a.token_tab_pos=1
		self.select_param_window_math_a.path_tab_pos=2

		self.select_param_window_math_b=select_param()
		self.select_param_window_math_b.init(self.tab_math)
		self.select_param_window_math_b.set_save_function(self.save_combo)
		self.select_param_window_math_b.file_name_tab_pos=4
		self.select_param_window_math_b.token_tab_pos=5
		self.select_param_window_math_b.path_tab_pos=6
		self.vbox.addWidget(self.tab_math)

		self.setLayout(self.vbox)

	def closeEvent(self, event):
		self.hide()

	def on_move_down(self):
		tab_move_down(self.tab_mm)
		self.save_combo()

	def on_move_up(self):
		tab_move_up(self.tab_mm)
		self.save_combo()
