# General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
# base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
# The model can simulate OLEDs, Perovskite cells, and OFETs.
# 
# Copyright (C) 2008-2020 Roderick C. I. MacKenzie
# All rights reserved.
# 
# https://www.gpvdm.com
# r.c.i.mackenzie at googlemail.com
# 
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the GPVDM nor the
#       names of its contributors may be used to endorse or promote products
#       derived from this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


import os
from inp import inp_load_file
from inp_util import inp_search_token_value
from inp import inp_save

from util import str2bool
from cal_path import get_sim_path

class segment():
	def __init__(self):
		self.name=""
		self.position=""
		self.active=False
		self.start=0.0
		self.depth=0.0
		self.voltage=0.0
		self.width=0.0

store=[]

def contacts_print():
	global store
	for s in store:
		print(s.start, s.width,s.depth,s.voltage,s.active)

def contacts_get_contacts():
	global store
	return len(store)

def contacts_get_array():
	global store
	return store

def contacts_clear():
	global store
	store=[]

def contacts_append(name,position,active,start,width,depth,voltage):
	global store
	s=segment()
	s.name=name
	s.position=position
	s.active=active
	s.start=start
	s.depth=depth
	s.voltage=voltage
	s.width=width
	store.append(s)

def contacts_dump():
	global store
	print("dump of contacts:")
	print(str(len(store)))
	i=0
	for s in store:
		print("#contact_name"+str(i))
		print(str(s.name))
		print("#contact_position"+str(i))
		print(str(s.position))
		print("#contact_active"+str(i))
		print(str(s.active))
		print("#contact_start"+str(i))
		print(str(s.start))
		print("#contact_width"+str(i))
		print(str(s.width))
		print("#contact_depth"+str(i))
		print(str(s.depth))
		print("#contact_voltage"+str(i))
		print(str(s.voltage))


def contacts_save():
	global store
	lines=[]
	lines.append("#contacts")
	lines.append(str(len(store)))
	i=0
	for s in store:
		lines.append("#contact_name"+str(i))
		lines.append(str(s.name))
		lines.append("#contact_position"+str(i))
		lines.append(str(s.position))
		lines.append("#contact_active"+str(i))
		lines.append(str(s.active))
		lines.append("#contact_start"+str(i))
		lines.append(str(s.start))
		lines.append("#contact_width"+str(i))
		lines.append(str(s.width))
		lines.append("#contact_depth"+str(i))
		lines.append(str(s.depth))
		lines.append("#contact_voltage"+str(i))
		lines.append(str(s.voltage))
		i=i+1
	lines.append("#ver")
	lines.append("1.1")
	lines.append("#end")

	inp_save(os.path.join(get_sim_path(),"contacts.inp"),lines)


def contacts_load():
	global store
	store=[]
	lines=[]
	pos=0
	lines=inp_load_file(os.path.join(get_sim_path(),"contacts.inp"))
	if lines!=False:
		pos=pos+1	#first comment
		layers=int(lines[pos])

		for i in range(0, layers):
			#name
			pos=pos+1					#token
			token=lines[pos]

			pos=pos+1
			name=lines[pos]			#read value

			#position
			pos=pos+1					#token
			token=lines[pos]

			pos=pos+1
			position=lines[pos]			#read value

			#active
			pos=pos+1					#token
			token=lines[pos]

			pos=pos+1
			active=lines[pos]			#read value

			#start
			pos=pos+1					#token
			token=lines[pos]

			pos=pos+1
			start=lines[pos]			#read value

			#width
			pos=pos+1					#token
			token=lines[pos]

			pos=pos+1
			width=lines[pos]			#read value

			#depth
			pos=pos+1					#token
			token=lines[pos]

			pos=pos+1
			depth=lines[pos]			#read value

			#voltage
			pos=pos+1					#token
			token=lines[pos]

			pos=pos+1
			voltage=lines[pos]			#read value


			contacts_append(name,position,str2bool(active),float(start),float(width),float(depth),float(voltage))

		#contacts_dump()

