# General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
# base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
# The model can simulate OLEDs, Perovskite cells, and OFETs.
# 
# Copyright (C) 2008-2020 Roderick C. I. MacKenzie
# All rights reserved.
# 
# https://www.gpvdm.com
# r.c.i.mackenzie at googlemail.com
# 
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the GPVDM nor the
#       names of its contributors may be used to endorse or promote products
#       derived from this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


import sys

try:
	from OpenGL.GL import *
	from OpenGL.GLU import *
	from PyQt5 import QtOpenGL
	from PyQt5.QtOpenGL import QGLWidget
	open_gl_ok=True
except:
	print("opengl error from gl_lib",sys.exc_info()[0])

import random
import os
from math import sqrt
from math import fabs
from lines import lines_read
from util import wavelength_to_rgb
from epitaxy import epitaxy_get_device_start
from util import isnumber

class fast_data():
	date=0
	m=0
	std=0
	out=[]

def fast_reset(d):
	d.date=0
	d.out=[]

ray_fast=fast_data()
fast_reset(ray_fast)

def fast_load(d,file_name):

	if os.path.isfile(file_name)==True:
		age = os.path.getmtime(file_name)

		if d.date!=age:
			d.out=[]
			if lines_read(d.out,file_name)==True:
				if len(d.out)==0:
					return False
				#print(d.out)
				d.date=age

				d.m=0
				s=0
				for i in range(0,len(d.out)):
					d.m=d.m+d.out[i].x
				d.m=d.m/len(d.out)

				for i in range(0,len(d.out)):
					s=s+(d.out[i].x-d.m)*(d.out[i].x-d.m)
				d.std=sqrt(s/len(d.out))

				return True
			else:
				return False

	return True

def draw_rays(ray_file,top,width,y_mul,w):
	global ray_fast
	d=ray_fast

	if fast_load(d,ray_file)==True:
		if len(d.out)>2:
			head, tail = os.path.split(ray_file)
			out=d.out
			m=d.m
			std=d.std

			#for i in range(0,len(out)):
			#	print(out[i].x,out[i].x)
			#print(ray_file)
			glLineWidth(2)
			num=tail[10:-4]
			if isnumber(num)==False:
				#print("not a number")
				return

			if std==0.0:
				#print("std is zero")
				return

			wavelength=float(num)
			r,g,b=wavelength_to_rgb(wavelength)

			glColor4f(r, g, b,0.5)
			glBegin(GL_QUADS)

			sub=epitaxy_get_device_start()
			s=0
			mm=0

			std_mul=0.05
			#print(len(d.out))
			#print(d.m)
			#print(d.std)
			x_mul=width/(std*std_mul)
			i=0
			#step=((int)(len(out)/6000))*2
			#if step<2:
			step=2

			while(i<len(out)-2):
				if fabs(out[i].x-m)<std*std_mul:
					if fabs(out[i+1].x-m)<std*std_mul:
						#print(sub)
						glVertex3f(width/2+(out[i].x-m)*x_mul, top-(out[i].y+sub)*y_mul, 0)
						glVertex3f(width/2+(out[i+1].x-m)*x_mul, top-(out[i+1].y+sub)*y_mul, 0)

						glVertex3f(width/2+(out[i+1].x-m)*x_mul, top-(out[i+1].y+sub)*y_mul, w)
						glVertex3f(width/2+(out[i].x-m)*x_mul, top-(out[i].y+sub)*y_mul, w)



				i=i+step

			glEnd()
