# General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
# base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
# The model can simulate OLEDs, Perovskite cells, and OFETs.
# 
# Copyright (C) 2008-2020 Roderick C. I. MacKenzie
# All rights reserved.
# 
# https://www.gpvdm.com
# r.c.i.mackenzie at googlemail.com
# 
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the GPVDM nor the
#       names of its contributors may be used to endorse or promote products
#       derived from this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


import os
#from progress import progress_class
#from process_events import process_events

#inp files
from inp import inp_load_file
from inp import inp_get_next_token_array
from inp import inp_isfile
from inp import inp_update_token_value
from inp_description import inp_file_to_description

#tabs
from tab_main import tab_main
from tab import tab_class

#util
from win_lin import running_on_linux

#paths
from cal_path import get_bin_path
from cal_path import get_exe_args

from code_ctrl import enable_webbrowser
from cal_path import get_exe_command
from epitaxy import epitaxy_print

#qt
from PyQt5.QtWidgets import QTabWidget,QWidget

#window
from tab_terminal import tab_terminal

if enable_webbrowser()==True:
	from information_webkit import information
else:
	from information_noweb import information

from help import help_window

import i18n
_ = i18n.language.gettext

from global_objects import global_object_register
from cal_path import get_sim_path

from tab_view import tab_view

from css import css_apply

class gpvdm_notebook(QTabWidget):
	#progress=progress_class()
	finished_loading=False
	item_factory=None
	menu_items=[]


	def __init__(self):
		QWidget.__init__(self)
		css_apply(self,"tab_default.css")
		self.terminal=None
		self.update_display_function=None
		self.currentChanged.connect(self.changed_click)
		global_object_register("notebook_goto_page",self.goto_page)

	def update(self):
		for i in range(0,self.count()):
			w=self.widget(i)
			w.update()

	def changed_click(self):

		if self.tabText(self.currentIndex()).strip()==_("Device structure"):
			help_window().help_set_help(["device.png",_("<big><b>The device structure tab</b></big><br> Use this tab to change the structure of the device, the layer thicknesses and to perform optical simulations.  You can also browse the materials data base and  edit the electrical mesh.")])

		if self.tabText(self.currentIndex()).strip()==_("Electrical parameters"):
			help_window().help_set_help(["tab.png",_("<big><b>The electrical parameters</b></big>\nThis tab contains the electrical model parameters, such as mobility, tail slope energy, and band gap.")])

		if self.tabText(self.currentIndex()).strip()==_("Terminal"):
			help_window().help_set_help(["utilities-terminal.png",_("<big><b>The terminal window</b></big>\nThe output of the model will be displayed in this window, watch this screen for debugging and convergence information.")])

		if self.tabText(self.currentIndex()).strip()==_("Information"):
			help_window().help_set_help(["help.png",_("<big><b>On-line help</b></big>\nYou can view the on-line help and manual here.")])

	def get_current_page(self):
		i=self.currentIndex()
		return self.tabText(i)


	def goto_page(self,page):
		self.blockSignals(True)
		for i in range(0,self.count()):
				if self.tabText(i)==page:
					self.setCurrentIndex(i)
					break
		self.blockSignals(False)

	def callback_switch_page(self, notebook, page, page_num):
		if self.last_page!=page_num:
			self.last_page=page_num
			self.get_children()[page_num].help()

	def clean_menu(self):
		for menu in self.menu_items:
			self.item_factory.delete_item(menu)
		self.menu_items=[]

	def callback_view_toggle(self, widget, data):
		self.toggle_tab_visible(data.get_label())

	def add_info_page(self):
		browser=information()
		self.addTab(browser,_("Information"))

	def load(self):
		self.clear()
		self.clean_menu()
		self.last_page=0

		#self.setTabsClosable(True)
		self.setMovable(True)
		if (os.path.isfile(os.path.join(get_sim_path(),"sim.gpvdm"))==True):
			self.finished_loading=False
			#self.progress.init()
			#self.progress.show()
			#self.progress.start()
			#self.progress.set_text("Loading..")
			#process_events()


#			dos_files=inp_get_token_value("device_epitaxy.inp", "#layers")

			widget=tab_main()
			self.addTab(widget,_("Device structure"))

			self.update_display_function=widget.update


			self.terminal=tab_terminal()
			self.terminal.init()
			self.addTab(self.terminal,_("Terminal"))
			self.terminal.run(os.getcwd(),get_exe_command()+" --version "+get_exe_args())
			global_object_register("terminal",self.terminal)

			widget=tab_view()
			self.addTab(widget,_("Output"))

			#self.add_info_page()

			return True


			self.finished_loading=True
			#self.progress.stop()
			#self.progress.set_fraction(0.0)
			self.goto_page(_("Device structure"))

		else:
			self.add_info_page()
			self.goto_page(_("Information"))
			return False



