# General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
# base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
# The model can simulate OLEDs, Perovskite cells, and OFETs.
# 
# Copyright (C) 2008-2020 Roderick C. I. MacKenzie
# All rights reserved.
# 
# https://www.gpvdm.com
# r.c.i.mackenzie at googlemail.com
# 
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the GPVDM nor the
#       names of its contributors may be used to endorse or promote products
#       derived from this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


import os
import locale
import gettext
from cal_path import get_lang_path
from inp import inp_get_token_value
from cal_path import get_inp_file_path
locale_path = get_lang_path()
config_path =os.path.join(get_inp_file_path(),"lang.inp")
file_lang=inp_get_token_value(config_path, "#lang",archive="base.gpvdm")
if file_lang==None:
	file_lang="auto"

if file_lang=="auto":
	current_locale, encoding = locale.getdefaultlocale()
	if current_locale==None:
		print("No local language set assuming en_US")
		current_locale="en_US"
else:
	current_locale=file_lang
language = gettext.translation ('gpvdm', locale_path, [current_locale] , fallback=True)
language.install()


def yes_no(a):
	if a.lower() in ("ja","yes", "true", "t", "1"):
		return True
	elif a.lower() in ("nein","no", "false", "f", "0"):
		return False
	else:
		return a

def get_language():
	lang=current_locale.split("_")[1].lower()
	return lang

def get_full_language():
	return current_locale

def get_full_desired_lang_path():
	return os.path.join(get_lang_path(),get_full_language(),"LC_MESSAGES")

def get_languages():
	langs=[]
	langs.append("en_US")
	path=get_lang_path()
	for my_dir in os.listdir(path):
		if os.path.isdir(os.path.join(path,my_dir))==True:
			langs.append(my_dir)

	return langs
