# General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
# base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
# The model can simulate OLEDs, Perovskite cells, and OFETs.
# 
# Copyright (C) 2008-2020 Roderick C. I. MacKenzie
# All rights reserved.
# 
# https://www.gpvdm.com
# r.c.i.mackenzie at googlemail.com
# 
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the GPVDM nor the
#       names of its contributors may be used to endorse or promote products
#       derived from this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#import sys
import os
#import shutil
from inp import inp_save
from inp import inp_load_file
from code_ctrl import enable_betafeatures
from scan_item import scan_item_add

from cal_path import get_sim_path
from util import str2bool

class mesh:
	layers=[]
	remesh=False

xlist=mesh()
ylist=mesh()
zlist=mesh()

class mlayer():
	def __init__(self):
		self.thick=0.0
		self.points=0
		self.mul=1.0
		self.left_right="left"


def mesh_add(vector,thick,points,mul,left_right):
	a=mlayer()
	a.thick=float(thick)
	a.points=float(points)
	a.mul=float(mul)
	a.left_right=left_right
	if vector=="x":
		global xlist
		xlist.layers.append(a)
	elif vector=="y":
		global ylist
		ylist.layers.append(a)
	elif vector=="z":
		global zlist
		zlist.layers.append(a)


def mesh_load(vector):
	file_name="mesh_"+vector+".inp"

	if vector=="x":
		mesh_clear_xlist()
	elif vector=="y":
		mesh_clear_ylist()
	elif vector=="z":
		mesh_clear_zlist()

	my_list=[]
	pos=0
	lines=inp_load_file(file_name)

	if lines!=False:
		if lines[pos]!="#remesh_enable":			#Check we are not trying to open an old version
			return False

		pos=pos+1	#first comment
		remesh=str2bool(lines[pos])
		pos=pos+1	#remesh

		if vector=="x":
			global xlist
			xlist.remesh=remesh
		elif vector=="y":
			global ylist
			ylist.remesh=remesh
		elif vector=="z":
			global zlist
			zlist.remesh=remesh

		pos=pos+1	#first comment
		mesh_layers=int(lines[pos])
		for i in range(0, mesh_layers):
			#thick
			pos=pos+1			#token
			token=lines[pos]
			pos=pos+1
			thick=lines[pos]	#length

			pos=pos+1			#token
			token=lines[pos]
			pos=pos+1
			points=lines[pos] 	#points

			pos=pos+1			#token
			token=lines[pos]
			pos=pos+1
			mul=lines[pos] 		#mul

			pos=pos+1			#token
			token=lines[pos]
			pos=pos+1
			left_right=lines[pos] 		#left_right

			mesh_add(vector,thick,points,mul,left_right)

	return True

def mesh_save(file_name,mesh_class):
	lines=[]
	lines.append("#remesh_enable")
	lines.append(str(mesh_class.remesh))
	lines.append("#mesh_layers")
	lines.append(str(len(mesh_class.layers)))
	i=0
	for item in mesh_class.layers:
		lines.append("#mesh_layer_length"+str(i))
		lines.append(str(item.thick))
		lines.append("#mesh_layer_points"+str(i))
		lines.append(str(item.points))
		lines.append("#mesh_layer_mul"+str(i))
		lines.append(str(item.mul))
		lines.append("#mesh_layer_left_right"+str(i))
		lines.append(str(item.left_right))
		i=i+1
	lines.append("#ver")
	lines.append("1.0")
	lines.append("#end")
	inp_save(file_name,lines)

def mesh_save_x():
	global xlist
	mesh_save("mesh_x.inp",xlist)

def mesh_save_y():
	global ylist
	mesh_save("mesh_y.inp",ylist)

def mesh_save_z():
	global zlist
	mesh_save("mesh_z.inp",zlist)

def mesh_save_all():
	global xlist
	global ylist
	global zlist
	mesh_save("mesh_x.inp",xlist)
	mesh_save("mesh_y.inp",ylist)
	mesh_save("mesh_z.inp",zlist)


def mesh_load_all():
	ret=True
	r=mesh_load_x()
	ret=ret and r
	r=mesh_load_y()
	ret=ret and r
	r=mesh_load_z()
	ret=ret and r
	return ret

def mesh_load_x():
	return mesh_load("x")

def mesh_load_y():
	return mesh_load("y")

def mesh_load_z():
	return mesh_load("z")

def mesh_set_xlen(value):
	global xlist
	if len(xlist.layers)==1:
		xlist.layers[0].thick=value
		return True
	else:
		return False

def mesh_set_zlen(value):
	global zlist
	if len(zlist.layers)==1:
		zlist.layers[0].thick=value
		return True
	else:
		return False


def mesh_get_xlen():
	global xlist
	tot=0.0
	for a in xlist.layers:
		tot=tot+a.thick
	return tot

def mesh_get_ylen():
	global ylist
	tot=0.0
	for a in ylist.layers:
		tot=tot+a.thick
	return tot

def mesh_get_zlen():
	global zlist
	tot=0.0
	for a in zlist.layers:
		tot=tot+a.thick
	return tot

def mesh_get_xpoints():
	global xlist
	tot=0.0
	for a in xlist.layers:
		tot=tot+a.points
	return tot

def mesh_get_ypoints():
	global ylist
	tot=0.0
	for a in ylist.layers:
		tot=tot+a.points
	return tot

def mesh_get_zpoints():
	global zlist
	tot=0.0
	for a in zlist.layers:
		tot=tot+a.points
	return tot

def mesh_get_xlayers():
	global xlist
	return len(xlist.layers)

def mesh_get_ylayers():
	global ylist
	return len(ylist.layers)

def mesh_get_zlayers():
	global zlist
	return len(zlist.layers)

def mesh_get_xmesh():
	global xlist
	return xlist

def mesh_get_ymesh():
	global ylist
	return ylist

def mesh_get_zmesh():
	global zlist
	return zlist

def mesh_clear_xlist():
	global xlist
	zlist.remesh=True
	xlist.layers=[]

def mesh_clear_ylist():
	global ylist
	zlist.remesh=True
	ylist.layers=[]

def mesh_clear_zlist():
	global zlist
	zlist.remesh=True
	zlist.layers=[]

def mesh_clear():
	mesh_clear_xlist()
	mesh_clear_ylist()
	mesh_clear_zlist()
