# General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
# base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
# The model can simulate OLEDs, Perovskite cells, and OFETs.
# 
# Copyright (C) 2008-2020 Roderick C. I. MacKenzie
# All rights reserved.
# 
# https://www.gpvdm.com
# r.c.i.mackenzie at googlemail.com
# 
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the GPVDM nor the
#       names of its contributors may be used to endorse or promote products
#       derived from this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


import os

#qt
from PyQt5.QtCore import QSize, Qt
from PyQt5.QtWidgets import QWidget,QLineEdit,QComboBox,QHBoxLayout,QPushButton,QLabel,QDialog,QVBoxLayout,QToolBar,QSizePolicy,QAction,QTabWidget,QTableWidget,QAbstractItemView, QMenuBar,QApplication
from PyQt5.QtGui import QPainter,QIcon,QImage

from icon_lib import QIcon_load

from PyQt5.QtCore import QSize, Qt

from inp import inp_load_file

import functools

class plot_dlg_class(QDialog):

	def populate_combo_box_using_input_file(self,combobox,input_file):
#		try:
		lines=[]
		lines=inp_load_file(os.path.join(self.path,input_file))
		combobox.clear()

		for i in range(0, len(lines)):
			lines[i]=lines[i].rstrip()
			if len(lines[i])>0:
				if lines[i][0]=="#":
					combobox.addItem(lines[i])
#		except:
#
#			combobox.clear()

	def callback_edit0(self):
		self.populate_combo_box_using_input_file(self.token0,self.file0.text())

	def callback_edit1(self):
		self.populate_combo_box_using_input_file(self.token1,self.file1.text())

	def callback_edit2(self):
		self.populate_combo_box_using_input_file(self.token2,self.file2.text())

	def callback_click_ok(self,data):
		data.file0=self.file0.text()
		data.tag0=str(self.token0.currentText())
		data.file1=self.file1.text()
		data.tag1=str(self.token1.currentText())
		data.file2=self.file2.text()
		data.tag2=str(self.token2.currentText())
		self.ret=True
		self.close()

	def callback_click_cancel(self):
		self.ret=False
		self.close()

	def __init__(self,data):
		QWidget.__init__(self)
		self.path=os.path.dirname(data.example_file0)
		self.setWindowIcon(QIcon_load("jv"))
		self.setWindowTitle(_("Steady state simulation (www.gpvdm.com)"))
		self.setWindowFlags(Qt.WindowStaysOnTopHint)

		l=QLabel("x-axis:")
		vbox=QVBoxLayout()
		hbox=QHBoxLayout()
		self.file0 = QLineEdit()
		self.file0.setText(data.file0)
		self.file0.textChanged.connect(self.callback_edit0)
		hbox.addWidget(self.file0)


		self.token0 = QComboBox()
		self.populate_combo_box_using_input_file(self.token0, os.path.basename(data.example_file0))
		hbox.addWidget(self.token0)

		vbox.addWidget(l)
		hbox_widget=QWidget()
		hbox_widget.setLayout(hbox)
		vbox.addWidget(hbox_widget)

		#y-axis
		l=QLabel("y-axis:")
		l.show()

		hbox=QHBoxLayout()
		self.file1 = QLineEdit()
		self.file1.setText(data.file0)
		self.file1.textChanged.connect(self.callback_edit1)
		self.file1.show()
		hbox.addWidget(self.file1)

		self.token1 = QComboBox()
		self.populate_combo_box_using_input_file(self.token1,os.path.basename(data.example_file1))
		hbox.addWidget(self.token1)

		vbox.addWidget(l)
		hbox_widget=QWidget()
		hbox_widget.setLayout(hbox)
		vbox.addWidget(hbox_widget)

		#label
		l=QLabel("label:")

		hbox=QHBoxLayout()
		self.file2 = QLineEdit( )
		self.file2.setText(data.file0)
		self.file2.textChanged.connect(self.callback_edit2)
		hbox.addWidget(self.file2)

		self.token2 = QComboBox()
		self.populate_combo_box_using_input_file(self.token2,os.path.basename(data.example_file1))
		hbox.addWidget(self.token2)

		vbox.addWidget(l)
		hbox_widget=QWidget()
		hbox_widget.setLayout(hbox)
		vbox.addWidget(hbox_widget)

		button_box=QHBoxLayout()

		button_cancel=QPushButton("Cancel", self)
		button_box.addWidget(button_cancel)
		button_cancel.clicked.connect(self.callback_click_cancel)

		button_ok=QPushButton("Ok", self)
		button_ok.clicked.connect(functools.partial(self.callback_click_ok,data))
		button_box.addWidget(button_ok)

		button_box_widget=QWidget()
		button_box_widget.setLayout(button_box)
		vbox.addWidget(button_box_widget)
		self.setLayout(vbox)


	def run(self):
		self.ret=False
		self.exec_()


