# General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
# base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
# The model can simulate OLEDs, Perovskite cells, and OFETs.
# 
# Copyright (C) 2008-2020 Roderick C. I. MacKenzie
# All rights reserved.
# 
# https://www.gpvdm.com
# r.c.i.mackenzie at googlemail.com
# 
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the GPVDM nor the
#       names of its contributors may be used to endorse or promote products
#       derived from this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


import os
from util_text import gkt_title_to_gnu_plot_title

def plot_export(dest_dir,input_files,state,fig):
	file_ext=os.path.splitext(dest_dir)[1]
	print(file_ext)

	if file_ext=="":
		print("Saving as tex file")
		if os.path.isdir(dest_dir)==False:
			os.makedirs(dest_dir)

		plot_file=dest_dir+'/plot.plot'

		#build plot script
		plot_string="set term postscript eps enhanced solid color \"Helvetica\" 28\n"
		plot_string=plot_string+"set key top left\n"
		plot_string=plot_string+"set format y \"%1.0t{/Symbol \\327}10^{%L}\"\n"
		plot_string=plot_string+"set format x \"%1.0t{/Symbol \\327}10^{%L}\"\n"
		plot_string=plot_string+"set xlabel '"+gkt_title_to_gnu_plot_title(state.x_label)+" ("+state.x_units+")'\n"
		plot_string=plot_string+"set ylabel '"+gkt_title_to_gnu_plot_title(state.data_label)+" ("+state.data_units+")'\n"
		#plot_string=plot_string+"set format y '%.1le%T'\n"

		if (state.logy==1):
			plot_string=plot_string+"set logscale y\n"

		if (state.logx==1):
			plot_string=plot_string+"set logscale x\n"

		plot_string=plot_string+"plot \\\n"
		for i in range(0, len(input_files)):
			line_name=os.path.basename(os.path.dirname(input_files[i]))
			plot_string=plot_string+"'"+input_files[i]+"' using ($1*"+str(state.x_mul)+"):($2*"+str(state.y_mul)+") with l lw 7 title '"+line_name+" "+state.key_units+"',\\\n"

		plot_string=plot_string[:-3]



		text_file = open(plot_file, "w")
		text_file.write(plot_string)
		text_file.close()

		#build make file
		makefile_string="main:\n"
		makefile_string=makefile_string+"\tgnuplot plot.plot >plot.eps\n"
		makefile_string=makefile_string+"\tgs -dNOPAUSE -r600 -dEPSCrop -sDEVICE=jpeg -sOutputFile=plot.jpg plot.eps -c quit\n"
		makefile_string=makefile_string+"\tgnome-open plot.jpg\n"

		text_file = open(dest_dir+"/makefile", "w")
		text_file.write(makefile_string)
		text_file.close()

	if file_ext==".png":
		print("Saving png as",dest_dir)
		fig.savefig(dest_dir)
