# General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
# base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
# The model can simulate OLEDs, Perovskite cells, and OFETs.
# 
# Copyright (C) 2008-2020 Roderick C. I. MacKenzie
# All rights reserved.
# 
# https://www.gpvdm.com
# r.c.i.mackenzie at googlemail.com
# 
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the GPVDM nor the
#       names of its contributors may be used to endorse or promote products
#       derived from this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


import os
from inp import inp_load_file
from inp import inp_get_token_value_from_list

class ref:
	def __init__(self):
		self.group=""
		self.author=""
		self.journal=""
		self.title=""
		self.volume=""
		self.pages=""
		self.year=""
		self.doi=""
		self.unformatted=""

def load_ref(file_name):
	r=ref()
	file_name=os.path.splitext(file_name)[0]+".ref"

	if os.path.isfile(file_name)==False:
		return None

	lines=inp_load_file(file_name)
	if lines!=False:
		text=""
		r.group=inp_get_token_value_from_list(lines, "#ref_research_group")
		if r.group==None:
			r.group=""

		r.author=inp_get_token_value_from_list(lines, "#ref_authors")
		if r.author==None:
			r.author=""

		r.journal=inp_get_token_value_from_list(lines, "#ref_jounral")
		if r.journal==None:
			r.journal=""

		r.title=inp_get_token_value_from_list(lines, "#ref_title")
		if r.title==None:
			r.title=""

		r.volume=inp_get_token_value_from_list(lines, "#ref_volume")
		if r.volume==None:
			r.volume=""

		r.pages=inp_get_token_value_from_list(lines, "#ref_pages")
		if r.pages==None:
			r.pages=""

		r.year=inp_get_token_value_from_list(lines, "#ref_year")
		if r.year==None:
			r.year=""

		r.doi=inp_get_token_value_from_list(lines, "#ref_doi")
		if r.doi==None:
			r.doi=""

		r.unformatted=inp_get_token_value_from_list(lines, "#ref_unformatted")
		if r.unformatted==None:
			r.unformatted=""

	return r

