# General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
# base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
# The model can simulate OLEDs, Perovskite cells, and OFETs.
# 
# Copyright (C) 2008-2020 Roderick C. I. MacKenzie
# All rights reserved.
# 
# https://www.gpvdm.com
# r.c.i.mackenzie at googlemail.com
# 
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the GPVDM nor the
#       names of its contributors may be used to endorse or promote products
#       derived from this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


import os
import shutil
import glob
import random
from random import randint

from inp import inp_update_token_value
from scan_item import scan_items_index_item
from inp import inp_get_token_value
from inp import inp_get_token_array
from util import str2bool
from util import get_cache_path
from cal_path import get_materials_path
from clone import clone_materials
from scan_item import scan_items_get_file
from scan_item import scan_items_get_token
from clone import gpvdm_clone
#windows
import codecs
from util_zip import archive_decompress
from util_zip import archive_compress

from cal_path import subtract_paths

from progress import progress_class
from process_events import process_events

copy_materials=False

def tree_load_flat_list(sim_dir):
	config=[]
	file_name=os.path.join(sim_dir,'flat_list.inp')

	if os.path.isfile(file_name)==False:
		return False

	f = open(file_name)
	lines = f.readlines()
	f.close()

	for i in range(0, len(lines)):
		lines[i]=lines[i].rstrip()

	number=int(lines[0])

	for i in range(1,number+1):
		lines[i]=os.path.join(sim_dir,lines[i])
		if os.path.isdir(lines[i]):
			config.append(lines[i])

	return config

def tree_save_flat_list(sim_dir,flat_list):
	config=[]
	file_name=os.path.join(sim_dir,'flat_list.inp')

	a = open(file_name, "w")
	a.write(str(len(flat_list))+"\n")
	for i in range(0,len(flat_list)):
		rel_dir=subtract_paths(sim_dir,flat_list[i])
		a.write(rel_dir+"\n")

	a.close()

	return config

def tree_gen_flat_list(dir_to_search,level=0):
	found_dirs=[]
	for root, dirs, files in os.walk(dir_to_search):
		for name in files:

			if name=="sim.gpvdm":
				full_name=os.path.join(root, name)
				f=subtract_paths(dir_to_search,full_name)
				f=os.path.dirname(f)
				if len(f.split("/"))>level:
					found_dirs.append(f)
	return found_dirs

def tree_load_program(sim_dir):

	program_list=[]

	file_name=os.path.join(sim_dir,'gpvdm_gui_config.inp')

	if os.path.isfile(file_name)==True:
		f=open(file_name)
		config = f.readlines()
		f.close()

		for ii in range(0, len(config)):
			config[ii]=config[ii].rstrip()

		pos=0
		mylen=int(config[0])
		pos=pos+1

		for i in range(0, mylen):
			program_list.append([config[pos],config[pos+1],config[pos+3], config[pos+4]])
			pos=pos+6
	return program_list

def tree_load_config(sim_dir):
	global copy_materials

	copy_materials=inp_get_token_value(os.path.join(sim_dir,"scan_config.inp"),"#copy_materials")
	if copy_materials==None:
		copy_materials="False"
	copy_materials=str2bool(copy_materials)


def build_scan_tree(program_list):
	tree_items=[[],[],[]]	#file,token,values,opp
	for i in range(0,len(program_list)):
		#print(i,program_list[i][0],program_list[i][1],program_list[i][2],program_list[i][3])
		if program_list[i][3]=="scan":
			tree_items[0].append(program_list[i][0])
			tree_items[1].append(program_list[i][1])
			values=program_list[i][2]
			#This expands a [ start stop step ] command.
			if len(values)>0:
				if values[0]=='[' and values[len(values)-1]==']':
					values=values[1:len(values)-1]
					data=values.split()
					if len(data)==3:
						a=float(data[0])
						b=float(data[1])
						c=float(data[2])
						values=""
						pos=a
						while pos<b:
							values=values+str(pos)+" "
							pos=pos+c
						values=values[0:len(values)-1]

			tree_items[2].append(values)

	return tree_items

def tree_gen(flat_simulation_list,program_list,base_dir,sim_dir):
	sim_dir=os.path.abspath(sim_dir)	# we are about to traverse the directory structure better to have the abs path
	#print("here",program_list)
	found_scan=False
	found_random=False
	for i in range(0,len(program_list)):
		if program_list[i][3]=="scan":
			found_scan=True

		if program_list[i][3]=="random_file_name":
			found_random=True

	if found_scan==True and found_random==True:
		return False

	if found_random==True:
		tree_gen_random_files(sim_dir,flat_simulation_list,program_list,base_dir)
		return

	tree_items=build_scan_tree(program_list)			#tree_items[3].append(program_list[i][3])

	ret=tree(flat_simulation_list,program_list,tree_items,base_dir,0,sim_dir,"","")

	return ret

def tree_apply_mirror(directory,program_list):
	for i in range(0, len(program_list)):
		#print(program_list[i])
		if program_list[i][2]=="mirror":
			f=scan_items_get_file(program_list[i][3])
			t=scan_items_get_token(program_list[i][3])

			#print(f,t,program_list[i][3])
			src_value=inp_get_token_value(os.path.join(directory,f), t)
			#print("mirror src",f,t,src_value)
			inp_update_token_value(os.path.join(directory,program_list[i][0]), program_list[i][1], src_value)
			#print("mirror to",os.path.join(directory,program_list[i][0]), program_list[i][1])
	return True

def tree_apply_constant(directory,program_list):
	for i in range(0, len(program_list)):
		if program_list[i][3]=="constant":
			inp_update_token_value(os.path.join(directory,program_list[i][0]), program_list[i][1], program_list[i][2])

	return True

def tree_apply_python_script(directory,program_list):
	for i in range(0, len(program_list)):
		if program_list[i][3]=="python_code":

			ret=""
			command=program_list[i][2]
			ret=eval(command)
			file_path=os.path.join(directory,program_list[i][0])
			#print("EXEC=",command,">",ret,"<")
			inp_update_token_value(file_path, program_list[i][1], ret)
			#print("Replace",file_path, program_list[i][1], ret)

	return True

def copy_simulation(base_dir,cur_dir):
	global copy_materials
	gpvdm_clone(cur_dir,src_archive=os.path.join(base_dir, "sim.gpvdm"),dest="file")

	#print(">>>>>>>>>>>>>>>>>>>>>>>materials>>>>>>>>",copy_materials)
	if copy_materials==True:
		clone_materials(cur_dir)

def tree_gen_random_files(sim_path,flat_simulation_list,program_list,base_dir):
	length=0

	for i in range(0,len(program_list)):
		if program_list[i][3]=="random_file_name":
			length=int(program_list[i][2])

	progress_window=progress_class()
	progress_window.show()
	progress_window.start()

	process_events()

	for i in range(0,length):
		rand=codecs.encode(os.urandom(int(16 / 2)), 'hex').decode()
		cur_dir=os.path.join(sim_path,rand)

		if not os.path.exists(cur_dir):
			os.makedirs(cur_dir)
			gpvdm_clone(cur_dir,src_archive=os.path.join(base_dir, "sim.gpvdm"),dest="file")

			os.chdir(cur_dir)
			archive_decompress(os.path.join(cur_dir,"sim.gpvdm"))
			if tree_apply_constant(cur_dir,program_list)==False:
				return False

			if tree_apply_python_script(cur_dir,program_list)==False:
				return False

			tree_apply_mirror(cur_dir,program_list)

			archive_compress(os.path.join(cur_dir,"sim.gpvdm"))

			flat_simulation_list.append(cur_dir)

			progress_window.set_fraction(float(i)/float(length))
			progress_window.set_text("Adding "+cur_dir)
			process_events()

	progress_window.stop()

def tree(flat_simulation_list,program_list,tree_items,base_dir,level,path,var_to_change,value_to_change):
	#print(level,tree_items)
	values=tree_items[2][level]
	values=values.split()

	if tree_items[0][level]=="notknown":
		return False

	if tree_items[1][level]=="notknown":
		return False

	pass_var_to_change=var_to_change+" "+str(level)
	#print(pass_var_to_change)
	for ii in values:
		cur_dir=os.path.join(path,ii)

		if not os.path.exists(cur_dir):
			os.makedirs(cur_dir)

		pass_value_to_change=value_to_change+" "+ii

		if ((level+1)<len(tree_items[0])):
				ret=tree(flat_simulation_list,program_list,tree_items,base_dir,level+1,cur_dir,pass_var_to_change,pass_value_to_change)
				if ret==False:
					return False
		else:
			flat_simulation_list.append(cur_dir)
			new_values=pass_value_to_change.split()
			pos=pass_var_to_change.split()

			config_file=os.path.join(cur_dir,"sim.gpvdm")
			if os.path.isfile(config_file)==False:	#Don't build a simulation over something that exists already
				copy_simulation(base_dir,cur_dir)

				os.chdir(cur_dir)
				if tree_apply_constant(cur_dir,program_list)==False:
					return False

				if tree_apply_python_script(cur_dir,program_list)==False:
					return False

				for i in range(0, len(pos)):
					file_path=os.path.join(cur_dir,tree_items[0][int(pos[i])])
					inp_update_token_value(file_path, tree_items[1][int(pos[i])], new_values[i])
					#print("updating", file_path, tree_items[1][int(pos[i])], new_values[i])

				tree_apply_mirror(cur_dir,program_list)

				inp_update_token_value(os.path.join(cur_dir,"dump.inp"), "#plot", "0")

		if level==0:
			f = open(os.path.join(cur_dir,'scan.inp'),'w')
			f.write("data")
			f.close()
	return True
