# General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
# base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
# The model can simulate OLEDs, Perovskite cells, and OFETs.
# 
# Copyright (C) 2008-2020 Roderick C. I. MacKenzie
# All rights reserved.
# 
# https://www.gpvdm.com
# r.c.i.mackenzie at googlemail.com
# 
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the GPVDM nor the
#       names of its contributors may be used to endorse or promote products
#       derived from this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


import os
from tab import tab_class

#qt
from PyQt5.QtCore import QSize, Qt
from PyQt5.QtWidgets import QWidget,QVBoxLayout,QToolBar,QSizePolicy,QAction,QTabWidget,QSlider,QHBoxLayout,QLabel,QComboBox
from PyQt5.QtGui import QPainter,QIcon
from PyQt5.QtCore import pyqtSignal

from help import help_window

from dat_file import dat_file
from dat_file import dat_file_read
from dat_file_math import dat_file_max_min


class snapshot_slider(QWidget):

	changed = pyqtSignal()

	def cal_min_max(self):

		self.z_max=-1e40
		self.z_min=1e40

		for i in range(0,len(self.dirs)):
			fname=os.path.join(self.dirs[i],self.files_combo.currentText())
			x=[]
			y=[]
			z=[]

			my_data=dat_file()
			if dat_file_read(my_data,fname) == True:
				#print(z)
				temp_max,temp_min=dat_file_max_min(my_data)

				if temp_max>self.z_max:
					self.z_max=temp_max

				if temp_min<self.z_min:
					self.z_min=temp_min

	def update(self):
		self.dirs=[]
		if os.path.isdir(self.path)==True:
			for name in os.listdir(self.path):
				if name!="." and name!= "..":
					full_path=os.path.join(self.path, name)
					if os.path.isdir(full_path):
						self.dirs.append(full_path)
		self.slider_max=len(self.dirs)-1
		self.slider0.setMaximum(self.slider_max)
		self.update_file_combo()

	def slider0_change(self):
		value = self.slider0.value()
		self.label0.setText(str(value))
		self.changed.emit()

	def get_file_name(self):
		file_path=os.path.join(self.path,str(self.slider0.value()),self.files_combo.currentText())
		if os.path.isfile(file_path)==False:
			file_path=None
		return file_path

	def set_path(self,path):
		self.path=path
		self.update()
		self.cal_min_max()


	def __init__(self):
		QWidget.__init__(self)
		self.path=""

		self.setWindowTitle(_("Snapshot slider"))

		self.main_vbox = QVBoxLayout()

		self.slider_hbox0= QHBoxLayout()
		self.slider_max=30

		self.slider0 = QSlider(Qt.Horizontal)
		self.slider0.setMinimum(10)
		self.slider0.setMaximum(self.slider_max)

		self.slider0.setTickPosition(QSlider.TicksBelow)
		self.slider0.setTickInterval(5)
		self.slider0.valueChanged.connect(self.slider0_change)
		self.slider0.setMinimumSize(300, 80)

		self.slider_hbox0.addWidget(self.slider0)

		self.label0 = QLabel()
		self.label0.setText("")

		self.slider0.setValue(20)

		self.slider_hbox0.addWidget(self.label0)

		self.widget0=QWidget()
		self.widget0.setLayout(self.slider_hbox0)

		self.main_vbox.addWidget(self.widget0)


################
		self.slider_hbox1= QHBoxLayout()
		self.label1 = QLabel()
		self.label1.setText(_("File"))
		self.slider_hbox1.addWidget(self.label1)

		self.files_combo=QComboBox()
		self.slider_hbox1.addWidget(self.files_combo)

		self.files_combo.currentIndexChanged.connect(self.files_combo_changed)

		self.widget1=QWidget()
		self.widget1.setLayout(self.slider_hbox1)

		self.main_vbox.addWidget(self.widget1)

###############

		self.setLayout(self.main_vbox)

	def update_file_combo(self):
		self.files_combo.blockSignals(True)
		self.files_combo.clear()
		path=os.path.join(self.path,str(self.slider0.value()))
		if os.path.isdir(path)==True:
			for name in os.listdir(path):
				full_path=os.path.join(path, name)
				if os.path.isfile(full_path):
					if name!="snapshot_info.dat":
						self.files_combo.addItem(name)

		all_items  = [self.files_combo.itemText(i) for i in range(self.files_combo.count())]

		for i in range(0,len(all_items)):
			if all_items[i] == "Jn.dat":
				self.files_combo.setCurrentIndex(i)
		self.files_combo.blockSignals(False)


	def files_combo_changed(self):
		self.cal_min_max()
		self.changed.emit()


