# General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
# base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
# The model can simulate OLEDs, Perovskite cells, and OFETs.
# 
# Copyright (C) 2008-2020 Roderick C. I. MacKenzie
# All rights reserved.
# 
# https://www.gpvdm.com
# r.c.i.mackenzie at googlemail.com
# 
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the GPVDM nor the
#       names of its contributors may be used to endorse or promote products
#       derived from this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


import sys
from PyQt5.QtWidgets import QMenuBar, QWidget, QApplication, QAction,QDesktopWidget,QTabWidget,QVBoxLayout
from PyQt5.QtGui import QIcon

import os

from solar_planet import planet
from ribbon_solar import ribbon_solar

from icon_lib import QIcon_load
from PyQt5.QtCore import pyqtSignal

class solar_main(QWidget):

	update = pyqtSignal()

	def __init__(self,path):
		self.path=path
		self.export_file_name=os.path.join(self.path,"spectra.inp")
		super().__init__()
		self.resize(1200,600)
		self.setWindowIcon(QIcon_load("weather-few-clouds"))

		self.vbox=QVBoxLayout()

		self.ribbon = ribbon_solar()
		self.vbox.addWidget(self.ribbon)

		self.ribbon.run.triggered.connect(self.callback_run)

		self.ribbon.export.triggered.connect(self.callback_export)

		self.setWindowTitle(_("Solar Spectrum Generator")+" (https://www.gpvdm.com)")
		self.center()

		self.notebook = QTabWidget()

		self.vbox.addWidget(self.notebook)

		earth = planet(self.export_file_name)
		earth.set_earth(True)
		earth.init()
		self.notebook.addTab(earth,"Earth")

		#mercury = planet(self.export_file_name)
		#mercury.set_mercury(True)
		#mercury.set_orbitalpoint(True)
		#mercury.init()
		#self.notebook.addTab(mercury,"Mercury")

		#venus = planet(self.export_file_name)
		#venus.set_venus(True)
		#venus.set_orbitalpoint(True)
		#venus.init()
		#self.notebook.addTab(venus,"Venus")

		#mars = planet(self.export_file_name)
		#mars.set_mars(True)
		#mars.set_orbitalpoint(True)
		#mars.init()
		#self.notebook.addTab(mars,"Mars")

		#ceres = planet(self.export_file_name)
		#ceres.set_ceres(True)
		#ceres.set_orbitalpoint(True)
		#ceres.init()
		#self.notebook.addTab(ceres, "Ceres (Dwarf Planet)")

		#europa = planet(self.export_file_name)
		#europa.set_europa(True)
		#europa.set_orbitalpoint(True)
		#europa.init()
		#self.notebook.addTab(europa, "Europa (moon of Jupiter)")

		#halley = planet(self.export_file_name)
		#halley.set_halley(True)
		#halley.set_orbitalpoint(True)
		#halley.init()
		#self.notebook.addTab(halley, "Halley's Comet")

		#pluto = planet(self.export_file_name)
		#pluto.set_pluto(True)
		#pluto.set_orbitalpoint(True)
		#pluto.init()
		#self.notebook.addTab(pluto, "Pluto")

		self.setLayout(self.vbox)


	def callback_run(self):
		tab = self.notebook.currentWidget()
		tab.update()

	def callback_export(self):
		tab = self.notebook.currentWidget()
		tab.export()
		self.update.emit()


	def save(self):
		self.notebook.currentWidget().save()

	def copy(self):
		self.notebook.currentWidget().copy2clip()

	def center(self):
		qr = self.frameGeometry()
		cp = QDesktopWidget().availableGeometry().center()
		qr.moveCenter(cp)
		self.move(qr.topLeft())


if __name__ == '__main__':
	app = QApplication(sys.argv)
	win = spectrum_main()
	sys.exit(app.exec_())
