# General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
# base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
# The model can simulate OLEDs, Perovskite cells, and OFETs.
# 
# Copyright (C) 2008-2020 Roderick C. I. MacKenzie
# All rights reserved.
# 
# https://www.gpvdm.com
# r.c.i.mackenzie at googlemail.com
# 
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the GPVDM nor the
#       names of its contributors may be used to endorse or promote products
#       derived from this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


import sys
import os

from win_lin import running_on_linux

import i18n
_ = i18n.language.gettext

from gui_enable import gui_get
if gui_get()==True:
	from PyQt5.QtWidgets import QSystemTrayIcon,QMenu,QApplication
	from about import about_dlg
	import webbrowser
	from icon_lib import QIcon_load

from cluster import cluster



statusicon = None

if gui_get()==True:
	class tray_icon(QSystemTrayIcon):

		def __init__(self,  parent=None):
			QSystemTrayIcon.__init__(self, QIcon_load("ball_green"), parent)
			menu = QMenu(parent)
			self.menu_about = menu.addAction(_("About"))
			self.menu_about.triggered.connect(self.callback_about)
			self.menu_man = menu.addAction(_("Manual"))
			self.menu_man.triggered.connect(self.callback_man)

			self.menu_youtube = menu.addAction("&"+_("Youtube channel"))
			self.menu_youtube.triggered.connect(self.callback_youtube)

			self.exitAction = menu.addSeparator()
			self.exitAction = menu.addAction(_("Quit"))

			self.exitAction.triggered.connect(self.callback_exit)
			self.setContextMenu(menu)

		def callback_exit(self):
			QApplication.quit()

		def callback_about(self):
			dlg=about_dlg()
			dlg.exec_()

		def callback_man(self):
			webbrowser.open("https://www.gpvdm.com/man.html")
		def	callback_youtube(self):
			webbrowser.open("https://www.youtube.com/channel/UCbm_0AKX1SpbMMT7jilxFfA")


def status_icon_init():
	if gui_get()==True:
		global statusicon
		statusicon=tray_icon()
		statusicon.show()

def status_icon_run(cluster):
	if gui_get()==True:
		global statusicon
		if cluster==False:
			statusicon.setIcon(QIcon_load("ball_red"))
		else:
			statusicon.setIcon(QIcon_load("ball_red4"))

def status_icon_stop(cluster):
	if gui_get()==True:
		global statusicon
		if cluster==False:
			statusicon.setIcon(QIcon_load("ball_green"))
		else:
			statusicon.setIcon(QIcon_load("ball_green4"))

def status_icon_get():
	if gui_get()==True:
		global statusicon
		return statusicon
