# General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
# base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
# The model can simulate OLEDs, Perovskite cells, and OFETs.
# 
# Copyright (C) 2008-2020 Roderick C. I. MacKenzie
# All rights reserved.
# 
# https://www.gpvdm.com
# r.c.i.mackenzie at googlemail.com
# 
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the GPVDM nor the
#       names of its contributors may be used to endorse or promote products
#       derived from this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


import os
from inp import inp_update_token_value
from inp import inp_get_token_value
import zipfile
import glob
from tab import tab_class

#qt
from PyQt5.QtCore import QSize, Qt
from PyQt5.QtWidgets import QWidget,QVBoxLayout,QToolBar,QSizePolicy,QAction,QTabWidget,QSystemTrayIcon,QMenu, QComboBox, QMenuBar, QLabel
from PyQt5.QtGui import QIcon

from cal_path import get_sim_path
from cal_path import get_plugins_path

class tb_optical_model(QWidget):

	def __init__(self):
		QWidget.__init__(self)
		self.dump_dir=os.path.join(get_sim_path(),"light_dump")

		self.layout=QVBoxLayout()
		label=QLabel(_("Optical model")+":")
		self.layout.addWidget(label)

		self.cb = QComboBox()
		self.cb.activated.connect(self.on_cb_model_changed)
		self.layout.addWidget(self.cb)
		self.update()
		self.setLayout(self.layout)
		self.show()

	def get_text(self):
		out=self.cb.currentText()
		out=out.split(" ")[0]
		return out

	def file_name_set_start(self,start):
		self.start=start

	def file_name_set_end(self,end):
		self.end=end

	def find_models(self):
		ret=[]
		path=get_plugins_path()

		for file in glob.glob(os.path.join(path,"*")):
			file_name=os.path.basename(file)
			if file_name.startswith("light_"):
				if file_name.endswith(".dll") or file_name.endswith(".so"):
					ret.append(os.path.basename(file_name[6:]).split(".")[0])

		return ret

	def on_cb_model_changed(self):
		cb_text=self.cb.currentText()
		inp_update_token_value("light.inp", "#light_model", cb_text)


	def update(self):
		self.cb.blockSignals(True)

		self.cb.clear()
		models=self.find_models()
		if len(models)==0:
			error_dlg(self,_("I can't find any optical plugins, I think the model is not installed properly."))
			return

		for i in range(0, len(models)):
			self.cb.addItem(models[i])

		used_model=inp_get_token_value(os.path.join(get_sim_path(),"light.inp"), "#light_model")
		print(models,used_model)
		if models.count(used_model)==0:
			used_model="exp"
			inp_update_token_value(os.path.join(get_sim_path(),"light.inp"), "#light_model","exp")
			self.cb.setCurrentIndex(self.cb.findText(used_model))
		else:
			self.cb.setCurrentIndex(self.cb.findText(used_model))

		self.cb.blockSignals(False)

