# General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
# base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
# The model can simulate OLEDs, Perovskite cells, and OFETs.
# 
# Copyright (C) 2008-2020 Roderick C. I. MacKenzie
# All rights reserved.
# 
# https://www.gpvdm.com
# r.c.i.mackenzie at googlemail.com
# 
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the GPVDM nor the
#       names of its contributors may be used to endorse or promote products
#       derived from this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


import os
from cal_path import get_inp_file_path
from cal_path import get_share_path
from cal_path import get_bin_path
from inp import inp_load_file
from inp import inp_update_token_value
from inp_util import inp_search_token_value
from util_zip import read_lines_from_archive

from cal_path import get_sim_path

global core
global ver_error
global subver

def ver_error():
	global ver_error
	return ver_error

def ver_core():
	global core
	return core

def ver_subver():
	global subver
	return subver

def version():
	global core
	global subver
	string="core: Version "+core+"."+subver
	return string

def ver():
	global core
	global subver
	return str(core)+"."+subver

def ver_load_info():
	lines=[]
	global core
	global ver_error
	global subver

	core=""
	ver_error=""


	ver_file_path=os.path.join(get_inp_file_path(),"ver.inp")

	lines=inp_load_file(ver_file_path,archive="base.gpvdm")
	if lines!=False:
		core=inp_search_token_value(lines,"#core")
		subver=inp_search_token_value(lines,"#sub_ver")
		return True
	else:
		ver_error="I can not find the file sim.gpvdm/ver.inp.\n\nI have tried looking in "+ver_file_path+"\n\nThe share path is"+get_share_path()+"\n\nThe bin path is"+get_bin_path()+"\n\nThe current working dir is "+get_sim_path()+"\n\nTry reinstalling a new version of gpvdm and/or report the bug to me at  roderick.mackenzie@nottingham.ac.uk."
		return False

def ver_sync_ver():
	file_name=os.path.join(os.getcwd(),"include","version.h")
	found=False

	if os.path.isfile(file_name)==True:
		f = open(file_name, "r")
		lines = f.readlines()
		f.close()
		for l in range(0, len(lines)):
			lines[l]=lines[l].rstrip()
			if lines[l].startswith("#define")==True:
				text=(lines[l].split("\t")[2].strip("\""))
				found=True

	if found==True:
		print("seting ver to: ",text)
		inp_update_token_value("ver.inp","#core",text,archive="sim.gpvdm")
		inp_update_token_value("ver.inp","#core",text,archive="base.gpvdm")

	else:
		print(_("version.h not found"))


def ver_check_compatibility(file_name):
	lines=[]
	core=""

	lines=read_lines_from_archive(file_name,"ver.inp")
	if lines!=False:
		core=inp_search_token_value(lines,"#core")
		if core==ver_core():
			return True
		else:
			return False

	return False


