// General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
// base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
// The model can simulate OLEDs, Perovskite cells, and OFETs.
// 
// Copyright (C) 2008-2020 Roderick C. I. MacKenzie
// All rights reserved.
// 
// https://www.gpvdm.com
// r.c.i.mackenzie at googlemail.com
// 
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the GPVDM nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include <stdio.h>
#include <stdlib.h>

	#include <dlfcn.h>

#include "util.h"
#include "inp.h"
#include "const.h"
#include "device.h"
#include "dump_ctrl.h"
#include "config.h"
#include "complex_solver.h"
#include "cal_path.h"
#include <lang.h>
#include <log.h>

static int unused __attribute__((unused));

void solver_init(struct simulation *sim,char *solver_name)
{
char lib_path[1000];

find_dll(sim, lib_path,solver_name);

char *error;

	sim->dll_matrix_handle = dlopen(lib_path, RTLD_LAZY);

	if (!sim->dll_matrix_handle)
	{
		ewe(sim, "%s\n", dlerror());
	}

	sim->dll_matrix_solve = dlsym(sim->dll_matrix_handle, "dll_matrix_solve");
	if ((error = dlerror()) != NULL)
	{
		ewe(sim, "%s\n", error);
	}

	sim->dll_matrix_dump = dlsym(sim->dll_matrix_handle, "dll_matrix_dump");
	if ((error = dlerror()) != NULL)
	{
		ewe(sim, "%s\n", error);
	}

	sim->dll_set_interface = dlsym(sim->dll_matrix_handle, "set_interface");
	if ((error = dlerror()) != NULL)
	{
		ewe(sim, "%s\n", error);
	}

	sim->dll_matrix_solver_free = dlsym(sim->dll_matrix_handle, "dll_matrix_solver_free");
	if ((error = dlerror()) != NULL)
	{
		ewe(sim, "%s\n", error);
	}



}


void solver(struct simulation *sim,int col,int nz,int *Ti,int *Tj, long double *Tx, long double *b)
{
(*sim->dll_matrix_solve)(sim,col,nz,Ti,Tj,Tx,b);
}

void dump_matrix(struct simulation *sim,struct device *in)
{
int i;
	for (i=0;i<in->N;i++)
	{
		printf_log(sim,"%ld %ld %Le\n",in->Ti[i],in->Tj[i],in->Tx[i]);
	}

	for (i=0;i<in->M;i++)
	{
		printf_log(sim,"%Le\n",in->b[i]);
	}
	//(*sim->dll_matrix_dump)(sim,col,nz,Ti,Tj, Tx,b,index);
}

void solver_free(struct simulation *sim)
{
if (sim->dll_matrix_handle!=NULL)
{
	(*sim->dll_matrix_solver_free)(sim);


	printf_log(sim,"%s=%p\n",_("Freeing memory"),sim->dll_matrix_handle);

	if (dlclose(sim->dll_matrix_handle)!=0)
	{
		ewe(sim,"%s\n",_("Error closing dll"));
	}

	sim->dll_matrix_handle=NULL;
	sim->dll_matrix_solve=NULL;
	sim->dll_matrix_dump=NULL;
	sim->dll_set_interface=NULL;
	sim->dll_matrix_solver_free=NULL;

}
}

