// General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
// base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
// The model can simulate OLEDs, Perovskite cells, and OFETs.
// 
// Copyright (C) 2008-2020 Roderick C. I. MacKenzie
// All rights reserved.
// 
// https://www.gpvdm.com
// r.c.i.mackenzie at googlemail.com
// 
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the GPVDM nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <sys/stat.h>
#include <unistd.h>
#include <dirent.h>
#include <fcntl.h>
#include "util.h"
#include "log.h"
#include <const.h>
#include <lang.h>
#include <math.h>

	#include <fnmatch.h>


int is_number(char a)
{
switch(a)
{
	case '1':
		return TRUE;
	case '2':
		return TRUE;
	case '3':
		return TRUE;
	case '4':
		return TRUE;
	case '5':
		return TRUE;
	case '6':
		return TRUE;
	case '7':
		return TRUE;
	case '8':
		return TRUE;
	case '9':
		return TRUE;
	case '0':
		return TRUE;
	case 'e':
		return TRUE;
	case 'E':
		return TRUE;
	case '+':
		return TRUE;
	case '-':
		return TRUE;
	case '.':
		return TRUE;
	default:
		return FALSE;
}

}

int get_number_in_string(double *out, char* in, int n)
{
	int i=0;
	int len=strlen(in);
	int m=0;
	int c=0;
	int number=-1;
	for (i=0;i<len;i++)
	{
		if (i>0)
		{
			m=is_number(in[i-1]);
		}else
		{
			m=FALSE;
		}

		c=is_number(in[i]);

		if ((m==FALSE) && (c==TRUE))
		{
			number++;
			if (number==n)
			{
				sscanf(&(in[i]),"%le",out);
				return 0;
			}
		}


	}

return -1;
}

int replace_number_in_string(char *buf, char* in, double replace, int n)
{
	int i=0;
	int len=strlen(in);
	int m=0;
	int c=0;
	int number=-1;
	int pos=0;
	strcpy(buf,"");

	for (i=0;i<len;i++)
	{
		if (i>0)
		{
		m=is_number(in[i-1]);
		}else
		{
			m=FALSE;
		}

		c=is_number(in[i]);

		if ((m==FALSE) && (c==TRUE))
		{
			number++;
			if (number==n)
			{
				char temp[200];
				sprintf(temp,"%le ",replace);
				strcat(buf,temp);
				pos=strlen(buf);
			}
		}

		if (number!=n)
		{
			buf[pos]=in[i];
			pos++;
			buf[pos]=0;

		}

	}

}

int fnmatch2(char *pat,char *in)
{

	return fnmatch(pat,in, FNM_PATHNAME);

}

void string_to_hex(char* out,char* in)
{
int i;
char temp[8];
strcpy(out,"");

for (i=0;i<strlen(in);i++)
{
	sprintf(temp,"%02x",in[i]);
	strcat(out,temp);
}

}


int cmpstr_min(char * in1,char *in2)
{
int i;
int max=strlen(in1);
if (strlen(in2)<max) max=strlen(in2);
for (i=0;i<max;i++)
{
	if (in1[i]!=in2[i]) return 1;
}
return 0;
}

int strextract_name(char *out,char * in)
{
int i;
for (i=0;i<strlen(in);i++)
{
	if (in[i]=='@')
	{
		out[i]=0;
		return strlen(out);
	}
	out[i]=in[i];

}
strcpy(out,"");
return -1;
}

int strcmp_end(char * str,char *end)
{
if (strlen(str)<strlen(end)) return 1;
int pos=strlen(str)-strlen(end);
return strcmp((char *)(str+pos),end);
}

int strcmp_begin(char * str,char *begin)
{
int i;
if (strlen(str)<strlen(begin)) return 1;
int lb=strlen(begin);
for (i=0;i<lb;i++)
{
	if (str[i]!=begin[i]) return 1;
}
return 0;
}

char* strextract_domain(char * in)
{
int i=0;
for (i=0;i<strlen(in)-1;i++)
{
	if (in[i]=='@')
	{
		return (char *)(&in[i+1]);
	}
}
return (char *)-1;
}

int is_domain(char * in)
{
int i=0;
for (i=0;i<strlen(in)-1;i++)
{
	if (in[i]=='@')
	{
		return 0;
	}
}


return -1;
}

int extract_str_number(char * in,char *cut)
{
int out;
int len=strlen(cut);
sscanf((in+len),"%d",&out);
return out;
}

int strextract_int(char * in)
{
char temp[200];
int i=0;
int ret=0.0;
int count=0;
for (i=0;i<strlen(in);i++)
{
	if ((in[i]>47)&&(in[i]<58))
	{
		temp[count]=in[i];
		count++;
	}

}
temp[count]=0;
sscanf(temp,"%d",&ret);
return ret;
}
