// General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
// base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
// The model can simulate OLEDs, Perovskite cells, and OFETs.
// 
// Copyright (C) 2008-2020 Roderick C. I. MacKenzie
// All rights reserved.
// 
// https://www.gpvdm.com
// r.c.i.mackenzie at googlemail.com
// 
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the GPVDM nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include <string.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <unistd.h>
#include <i.h>
#include <exp.h>
#include <dos.h>
#include "sim.h"
#include "dump.h"
#include "buffer.h"

#include "memory.h"
#include "contacts.h"
#include <lang.h>
#include <cal_path.h>
#include "contacts_vti_store.h"

static int unused __attribute__((unused));

void dump_contacts_init(struct simulation *sim,struct device *in,struct contacts_vti_store *store)
{
	if (in->ncontacts>2)
	{
		int i=0;

		for (i=0;i<in->ncontacts;i++)
		{
			inter_init(sim,&(store->i[i]));
		}
	}
}

void dump_contacts_save(struct simulation *sim,struct device *in,struct contacts_vti_store *store)
{
	if (in->ncontacts>2)
	{
		int i;
		int sub=TRUE;
		char temp[200];
		struct buffer buf;
		buffer_init(&buf);
		for (i=0;i<in->ncontacts;i++)
		{
			buffer_malloc(&buf);
			buf.y_mul=1.0;
			buf.x_mul=1e6;
			sprintf(buf.title,"%s",_("Voltage - Current"));
			strcpy(buf.type,"xy");
			strcpy(buf.x_label,_("Voltage"));
			strcpy(buf.x_units,"V");
			strcpy(buf.data_label,_("Current"));
			strcpy(buf.data_units,"A m^{-2}");
			buf.logscale_x=0;
			buf.logscale_y=0;
			buf.x=1;
			buf.y=store->i[i].len;
			buf.z=1;
			buffer_add_info(sim,&buf);
			buffer_add_xy_data(sim,&buf,store->i[i].x, store->i[i].data, store->i[i].len);
			sprintf(temp,"contact%d_i.dat",i);
			buffer_dump_path(sim,sim->output_path,temp,&buf);
			buffer_free(&buf);
		}
	}
}

void dump_contacts_add_data(struct simulation *sim,struct device *in,struct contacts_vti_store *store)
{
	if (in->ncontacts>2)
	{
		int i=0;
		gdouble x_value=0.0;
		for (i=0;i<in->ncontacts;i++)
		{
			x_value=contact_get_active_contact_voltage(sim,in);
	//		inter_append(&(store->v),x_value,contact_get_voltage(sim,in,i));
			inter_append(&(store->i[i]),x_value,contacts_get_J(in,i));
		}

	}
}

void dump_contacts_free(struct simulation *sim,struct device *in,struct contacts_vti_store *store)
{
	if (in->ncontacts>2)
	{
		int i=0;

		for (i=0;i<in->ncontacts;i++)
		{
			//inter_free(&(store->v[i]));
			inter_free(&(store->i[i]));
		}
	}
}

