// General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
// base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
// The model can simulate OLEDs, Perovskite cells, and OFETs.
// 
// Copyright (C) 2008-2020 Roderick C. I. MacKenzie
// All rights reserved.
// 
// https://www.gpvdm.com
// r.c.i.mackenzie at googlemail.com
// 
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the GPVDM nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include <util.h>
#include <dump_ctrl.h>
#include <complex_solver.h>
#include <const.h>
#include <light.h>
#include <device.h>
#include <light_interface.h>

#include <functions.h>
#include <log.h>


EXPORT void light_dll_ver(struct simulation *sim)
{
        printf_log(sim,"Exponential light model\n");
}

EXPORT int light_dll_solve_lam_slice(struct simulation *sim,struct light *in,int lam)
{
if (in->sun_E[lam]==0.0)
{
	return 0;
}

if (get_dump_status(sim,dump_optics)==TRUE)
{
	char one[100];
	sprintf(one,"Solve light optical slice at %Lf nm\n",in->l[lam]*1e9);

	waveprint(sim,one,in->l[lam]*1e9);
}

int i;

gdouble complex n0=0.0+0.0*I;

//complex gdouble r=0.0+0.0*I;
complex gdouble t=0.0+0.0*I;
gdouble complex beta0=0.0+0.0*I;
complex gdouble Ep=in->sun_E[lam]+0.0*I;
complex gdouble En=0.0+0.0*I;
gdouble dx=in->x[1]-in->x[0];

for (i=0;i<in->points;i++)
{

	n0=in->nbar[lam][i];
	beta0=(2*PI*n0/in->l[lam]);
	Ep=Ep*cexp(-beta0*dx*I);

	t=in->t[lam][i];



	in->Ep[lam][i]=creal(Ep);
	in->Epz[lam][i]=cimag(Ep);
	in->En[lam][i]=creal(En);
	in->Enz[lam][i]=cimag(En);

	if (i!=(in->points-1))
	{
		if ((in->n[lam][i]!=in->n[lam][i+1])||(in->alpha[lam][i]!=in->alpha[lam][i+1]))
		{
			Ep=Ep*t;
		}
	}
}

return 0;
}


