// General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
// base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
// The model can simulate OLEDs, Perovskite cells, and OFETs.
// 
// Copyright (C) 2008-2020 Roderick C. I. MacKenzie
// All rights reserved.
// 
// https://www.gpvdm.com
// r.c.i.mackenzie at googlemail.com
// 
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the GPVDM nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include <sim.h>
#include <dump.h>
#include <ntricks.h>
#include <dump.h>
#include <inp.h>
#include <buffer.h>
#include <light_interface.h>
#include <dynamic_store.h>
#include <light.h>
#include <buffer.h>
#include "photon_extraction_plugin.h"
#include <device.h>
#include <cal_path.h>

static int unused __attribute__((unused));

void sim_optics(struct simulation *sim,struct device *in)
{

	set_dump_status(sim,dump_lock, FALSE);
	//set_dump_status(sim,dump_optics, TRUE);
	//set_dump_status(sim,dump_optics_verbose, TRUE);
	set_dump_status(sim,dump_optics_summary, TRUE);

	struct light two;
	light_init(&two);

	struct buffer buf;
	buffer_init(&buf);

	struct inp_file inp;


	light_load_config(sim,&two);
	light_setup_dump_dir(sim,&two);

	strcpy(two.mode,"ray");
	//two.Psun=1.0;
	light_load_dlls(sim,&two);
	light_setup_ray(sim,in,&two);
	two.my_image.theta_steps=360;
	//light_set_dx(&cell.mylight,cell.ymesh[1]-cell.ymesh[0]);


	two.disable_transfer_to_electrical_mesh=TRUE;

	inp_init(sim,&inp);

	//inp_load_from_path(sim,&inp,get_input_path(sim),"light.inp");
	//inp_search_gdouble(sim,&inp,&(Psun),"#Psun");
	//Psun=fabs(Psun);
	light_set_sun(&two,1.0);

	two.force_update=TRUE;

	light_set_unity_power(&two);
	light_solve_all(sim,&two);


	buffer_malloc(&buf);
	buf.y_mul=1.0;
	buf.x_mul=1e9;
	strcpy(buf.title,"Photon escape probability");
	strcpy(buf.type,"xy");
	strcpy(buf.x_label,"Wavelength");
	strcpy(buf.data_label,"Probability");
	strcpy(buf.x_units,"nm");
	strcpy(buf.data_units,"a.u.");
	buf.logscale_x=0;
	buf.logscale_y=0;
	buf.x=1;
	buf.y=two.lpoints;
	buf.z=1;
	buffer_add_info(sim,&buf);
	buffer_add_xy_data(sim,&buf,two.l, two.extract_eff, two.lpoints);
	buffer_dump_path(sim,two.dump_dir,"light_escape_probability.dat",&buf);
	buffer_free(&buf);

	light_dump(sim,&two);
	light_dump_summary(sim,&two);

	light_free(sim,&two);



}
