# General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
# base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
# The model can simulate OLEDs, Perovskite cells, and OFETs.
# 
# Copyright (C) 2008-2020 Roderick C. I. MacKenzie
# All rights reserved.
# 
# https://www.gpvdm.com
# r.c.i.mackenzie at googlemail.com
# 
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the GPVDM nor the
#       names of its contributors may be used to endorse or promote products
#       derived from this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


import os
import sys
import shutil

def copy_lib(dest,src,name,mode="gpl_distro"):
	dest=os.path.join(dest,name)
	src=os.path.join(src,name)
	copy_unifdef(dest,src,[".c",".h",".inp"],mode=mode)
	copy_files(dest,src,[".po",".pot",".ref",".rc","Makefile.am"])

def copy_plugin(dest,src,name,mode="gpl_distro"):
	dest=os.path.join(dest,"plugin",name)
	src=os.path.join(src,"plugin",name)
	copy_unifdef(dest,src,[".c",".h",".inp"],mode=mode)
	copy_files(dest,src,[".po",".pot",".ref",".rc","Makefile.am"])

def copy_unifdef(dest,src,file_name_array,mode="gpl_distro"):
	if type(file_name_array)==str:
		file_name_array=[file_name_array]

	if os.path.isdir(dest)==False:
		os.mkdir(dest)

	for item in os.listdir(src):
		for file_name in file_name_array:
			if item.endswith(file_name)==True:
				src_file=os.path.join(src,item)
				dest_file=os.path.join(dest,item)

				if mode == "windows":
					os.system("unifdef -Denable_multi_layers -Dprivate  -Denable_clever_contacts -Denable_time -Denable_left_right_bias -Uenable_dos_an -Uenable_interface -Ufull_time_domain -Uenable_server -Uenable_remesh -Uenable_fit "+src_file+" > "+dest_file)
				if mode == "gpl_distro":
					os.system("unifdef -Denable_multi_layers -Uprivate  -Uenable_clever_contacts  -Denable_time -Uenable_left_right_bias -Uenable_dos_an -Uenable_interface -Ufull_time_domain -Uenable_server -Uenable_remesh -Uwindows -Uenable_fit "+src_file+" > "+dest_file)

def copy_files(dest,src,file_name_array):
	if type(file_name_array)==str:
		file_name_array=[file_name_array]

	if os.path.isdir(dest)==False:
		os.mkdir(dest)

	for item in os.listdir(src):
		for file_name in file_name_array:
			if item.endswith(file_name)==True:
				src_file=os.path.join(src,item)
				dest_file=os.path.join(dest,item)
				print("src=",src_file)
				print("dest=",dest_file)
				shutil.copy2(src_file, dest_file)
