# General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
# base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
# The model can simulate OLEDs, Perovskite cells, and OFETs.
# 
# Copyright (C) 2008-2020 Roderick C. I. MacKenzie
# All rights reserved.
# 
# https://www.gpvdm.com
# r.c.i.mackenzie at googlemail.com
# 
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the GPVDM nor the
#       names of its contributors may be used to endorse or promote products
#       derived from this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


import os
import sys
import argparse

def make_m4(hpc=False, win=False,usear=False):
	config_files=[]
	link_libs=""

	config_files.append("")

	config_files.append("lang")

	config_files.append("libi")
	link_libs=link_libs+" -lgpvdm_i"

	config_files.append("librpn")
	link_libs=link_libs+" -lgpvdm_rpn"

	config_files.append("libmemory")
	link_libs=link_libs+" -lgpvdm_memory"

	config_files.append("libdos")
	link_libs=link_libs+" -lgpvdm_dos"

	config_files.append("liblight")
	link_libs=link_libs+" -lgpvdm_light"

	config_files.append("libmeasure")
	link_libs=link_libs+" -lgpvdm_measure"

	config_files.append("libcontacts")
	link_libs=link_libs+" -lgpvdm_contacts"

	config_files.append("lib")
	link_libs=link_libs+" -lgpvdm_lib"

	config_files.append("libdump")
	link_libs=link_libs+" -lgpvdm_dump"

	config_files.append("libdumpctrl")
	link_libs=link_libs+" -lgpvdm_dumpctrl"

	config_files.append("libserver")
	link_libs=link_libs+" -lgpvdm_server"

	config_files.append("libmesh")
	link_libs=link_libs+" -lgpvdm_mesh"


	if win==False:
		if os.path.isdir("libfit"):
			config_files.append("libfit")
			link_libs=link_libs+" -lgpvdm_fit"

	for root, dirs, files in os.walk("./plugins"):
		for file in files:
			if file.endswith("Makefile.am"):
				name=os.path.join(root, file)[2:-12]
				config_files.append(name)

	config_files.append("src")
	if hpc==False:
		config_files.append("images/16x16")
		config_files.append("images/32x32")
		config_files.append("images/48x32")
		config_files.append("images/64x64")
		config_files.append("css")
		config_files.append("html")
		config_files.append("cluster")
		if win==False:
			config_files.append("man")

	f = open("config_files.m4", "w")
	for i in range(0,len(config_files)):
		f.write( "AC_CONFIG_FILES(["+os.path.join(config_files[i],"Makefile")+"])\n")

	f.close()

	f = open("make_files.m4", "w")
	f.write( "AC_SUBST(BUILD_DIRS,\"")
	for i in range(0,len(config_files)):
		f.write(config_files[i]+" ")

	f.write("\")")

	f.close()


	f = open("local_link.m4", "w")
	f.write( "AC_SUBST(LOCAL_LINK,\"")
	f.write(link_libs)
	f.write("\")")

	f.close()

	f = open("ar.m4", "w")

	if usear==True:
		f.write("AM_PROG_AR")
	else:
		f.write("")

	f.close()


if __name__ == "__main__":
	parser = argparse.ArgumentParser()
	parser.add_argument("--noar", help="no archiver for old Cent OS", action='store_true')
	parser.add_argument("--hpc", help="Set up for Nottingham hpc", action='store_true')
	parser.add_argument("--win", help="compile for windows", action='store_true')
	parser.add_argument("--verbosity", help="increase output verbosity", action='store_true')

	args = parser.parse_args()

	hpc=False
	win=False
	usear=True

	if args.hpc:
		hpc=True

	if args.win:
		win=True

	if args.noar:
		usear=False

	make_m4(hpc=hpc, win=win,usear=True)



